
/*****************************************************************************
** Internal: ProcessBreak()
**
** Aborts the entire installation process.
*/

ERROR ProcessBreak(struct Install *Self, struct XMLTag *Tag)
{
   DPrintF("Activate:","[Install] Processing <break> tag.");

   TerminateInstall(Self);

   return(ERR_Okay);
}

/*****************************************************************************
** Internal: ProcessCheck()
**
** Checks for the existence of a file and responds according to the script's
** requirements.
*/

ERROR ProcessCheck(struct Install *Self, struct XMLTag *Tag)
{
   struct XMLTag *tag;
   WORD i;
   LONG type;
   STRING file;
   BYTE onfail, pass;

   DPrintF("~Activate:","[Install] Processing <check> tag.");

   file = NULL;
   onfail = FALSE;
   pass = FALSE;
   for (i=1; i < Tag->TotalAttributes; i++) {
      if (StrCompare("file", Tag->Attributes[i].Name, 0, STR_MATCHLENGTH) IS ERR_Okay) {
         file = Tag->Attributes[i].Value;
      }
      else if (StrCompare("onfail", Tag->Attributes[i].Name, 0, STR_MATCHLENGTH) IS ERR_Okay) {
         onfail = TRUE;
      }
   }

   if (file) {
      if (AnalyseLocation(file, &type) IS ERR_Okay) {
         pass = TRUE;
      }
   }

   if (((pass IS TRUE) AND (onfail IS FALSE)) OR
       ((pass IS FALSE) AND (onfail IS TRUE))) {
      /*** Execute the content inside of our <check> tag ***/

      if (Tag->Child) {
         tag = Self->prvTag;
         Self->prvTag = Tag->Child;

         RunTags(Self);

         if (Self->prvTags) Self->prvTag = tag; /* Revert to the original tag position, assuming a break or error hasn't occurred */
      }

      StepBack();

      if (TotalChildren(Self->ViewID) > 0) return(ERR_Okay);
      else return(ERR_NoData);
   }
   else {
      StepBack();
      return(ERR_NoData);
   }
}

/*****************************************************************************
** Internal: ProcessComponents()
**
** If the developer has split the installation into packages then the component
** tag will be used to specify what those packages are.  The user can then
** select what components can be installed through check boxes.
*/

ERROR ProcessComponents(struct Install *Self, struct XMLTag *XMLTag)
{
   struct XMLTag *tag;
   OBJECTPTR text, line;
   UBYTE header[] = "<item><b><font colour=\"#202080\">Component Selection</font></b></item><item/><item>The following components are additional to the software installation and completely optional.  Recommended components are pre-selected for you.</item>";
   UBYTE buffer[256];
   LONG i, len;
   ERROR error;

   DPrintF("~Activate:","[Install] Processing <components> tag.");

   if (CreateObject(ID_TEXT, NULL, &text, NULL,
      FID_Owner|TLONG,        Self->ViewID,
      FID_LeftMargin|TLONG,   3,
      FID_RightMargin|TLONG,  3,
      FID_TopMargin|TLONG,    3,
      FID_BottomMargin|TLONG, 3,
      FID_Face|TSTRING,       glFontFace,
      FID_Flags|TLONG,        TXF_WORDWRAP,
      TAGEND) != ERR_Okay) {
      StepBack();
      return(ERR_CreateObject);
   }

   acDataChannel(text, Self->Head.UniqueID, DATA_XML, 1, header, sizeof(header), 1);

   /*** Create check boxes for each component ***/

   Self->prvOutput        = text;
   Self->prvOptionCount   = 0;
   Self->prvCurrentXCoord = 23;
   Self->prvCurrentYCoord = 90;
   Self->prvIndentLevel   = 0;
   Self->prvIndent        = 0;

   for (tag=XMLTag->Child; tag; tag=tag->Next) {
      for (i=0; glInstallCommands[i].Tag; i++) {
         if (StrCompare(glInstallCommands[i].Tag, tag->Attributes[0].Name, 0, STR_MATCHLENGTH) IS ERR_Okay) {
            if ((error = glInstallCommands[i].Routine(Self, text, tag)) != ERR_Okay) {
               len = StrFormat(buffer, sizeof(buffer), "<item/><item>An error occurred while installing package components (error code %d).  Install cancelled.</item>", error);
               acDataChannel(Self->prvOutput, Self->Head.UniqueID, DATA_XML, 1, buffer, len, 1);

               StepBack();
               return(ERR_False);
            }
            break;
         }
      }
   }

   /*** Draw lines to illustrate indentations in the option tree ***/

   for (i=0; i < Self->prvOptionCount; i++) {
      if (Self->prvOptions[i].Level > 0) {
         if (Self->prvOptions[i-1].Level < Self->prvOptions[i].Level) {
            CreateObject(ID_LINE, NULL, &line, NULL,
               FID_Owner|TLONG, Self->ViewID,
               FID_XCoord|TLONG,    Self->prvOptions[i].XCoord - Self->prvIndent + (Self->prvBoxSize/2),
               FID_YCoord|TLONG,    Self->prvOptions[i].YCoord - 9,
               FID_XEnd|TLONG,      Self->prvOptions[i].XCoord - Self->prvIndent + (Self->prvBoxSize/2),
               FID_YEnd|TLONG,      Self->prvOptions[i].YCoord + (Self->prvBoxSize/2),
               FID_Colour|TSTRING,  "#a0a0a0",
               TAGEND);
         }
         else {
            CreateObject(ID_LINE, NULL, &line, NULL,
               FID_Owner|TLONG, Self->ViewID,
               FID_XCoord|TLONG,    Self->prvOptions[i].XCoord - Self->prvIndent + (Self->prvBoxSize/2),
               FID_YCoord|TLONG,    Self->prvOptions[i].YCoord - (Self->prvBoxSize + 10),
               FID_XEnd|TLONG,      Self->prvOptions[i].XCoord - Self->prvIndent + (Self->prvBoxSize/2),
               FID_YEnd|TLONG,      Self->prvOptions[i].YCoord + (Self->prvBoxSize/2),
               FID_Colour|TSTRING,  "#a0a0a0",
               TAGEND);
         }

         CreateObject(ID_LINE, NULL, &line, NULL,
            FID_Owner|TLONG, Self->ViewID,
            FID_XCoord|TLONG,    Self->prvOptions[i].XCoord - Self->prvIndent + (Self->prvBoxSize/2),
            FID_YCoord|TLONG,    Self->prvOptions[i].YCoord + (Self->prvBoxSize/2),
            FID_XEnd|TLONG,      Self->prvOptions[i].XCoord - 1,
            FID_YEnd|TLONG,      Self->prvOptions[i].YCoord + (Self->prvBoxSize/2),
            FID_Colour|TSTRING,  "#a0a0a0",
            TAGEND);
      }
   }

   /*** Change the size of the page view to match the image ***/

   SetPage(Self, Self->prvCurrentYCoord);

   StepBack();
   return(ERR_Okay);
}

/*****************************************************************************
** Internal: ProcessDirectories()
**
** The directories tag is used to specify directory locations prior to
** installation.  This code will generate a set of input boxes and browse
** buttons so that the user can select destination directories.
**
** EXAMPLE
** -------
** <directories>
**   <destination path="programs:system/terminals/">Please select a destination for the ZTerm program.</destination>
**   <destination path="pandora_sdk:source/programs/zterm/">Please select a destination for the ZTerm source code.</destination>
** </directories>
*/

ERROR ProcessDirectories(struct Install *Self, struct XMLTag *XMLTag)
{
   struct XMLTag *tag;
   OBJECTPTR text;
   UBYTE header[] = "<item><b><font colour=\"#202080\">Directory Selection</font></b></item><item/><item>The following directories will be used for installing the software.  You may change them if you wish.</item>";
   UBYTE buffer[256];
   LONG i, len;
   ERROR error;

   DPrintF("~Activate:","[Install] Processing <directories> tag.");

   /*** Print title and introductory text ***/

   if (CreateObject(ID_TEXT, NULL, &text, NULL,
      FID_Owner|TLONG,        Self->ViewID,
      FID_LeftMargin|TLONG,   3,
      FID_RightMargin|TLONG,  3,
      FID_TopMargin|TLONG,    3,
      FID_BottomMargin|TLONG, 3,
      FID_Face|TSTRING,       glFontFace,
      FID_Flags|TLONG,        TXF_WORDWRAP,
      TAGEND) != ERR_Okay) {
      StepBack();
      return(ERR_CreateObject);
   }

   ActionTags(AC_DataChannel, text, Self->Head.UniqueID, DATA_XML, 1, header, sizeof(header), 1);

   /*** Create input boxes for each directory ***/

   Self->prvOutput         = text;
   Self->prvDirectoryCount = 0;
   Self->prvCurrentXCoord  = 3;
   Self->prvCurrentYCoord  = 70;

   /*** Process instructions within the tag ***/

   for (tag=XMLTag->Child; tag; tag=tag->Next) {
      for (i=0; glInstallCommands[i].Tag; i++) {
         if (StrCompare(glInstallCommands[i].Tag, tag->Attributes[0].Name, 0, STR_MATCHLENGTH) IS ERR_Okay) {
            if ((error = glInstallCommands[i].Routine(Self, text, tag)) != ERR_Okay) {
               len = StrFormat(buffer, sizeof(buffer), "<item/><item>An error occurred while creating program directories (error code %d).  Install cancelled.</item>", error);
               acDataChannel(Self->prvOutput, Self->Head.UniqueID, DATA_XML, 1, buffer, len, 1);

               SetPage(Self, Self->prvCurrentYCoord);
               StepBack();
               return(ERR_False);
            }
            break;
         }
      }
   }

   SetPage(Self, Self->prvCurrentYCoord);

   StepBack();
   if (Self->prvDirectoryCount <= 0) return(ERR_NoData);
   else return(ERR_Okay);
}

/*****************************************************************************
** Internal: ProcessImage()
**
** When an image tag is found, the image will be loaded from the compressed
** object and displayed in the viewing window.  This can be useful for
** creating splash images for example.
*/

ERROR ProcessImage(struct Install *Self, struct XMLTag *Tag)
{
   DPrintF("Activate:","[Install] Processing <image> tag.");
   return(ERR_Okay);
}

/*****************************************************************************
** Internal: ProcessInfo()
*/

struct InfoTag {
   STRING Tag;
   STRING Text;
};

struct InfoTag InfoArray[] = {
   { "name",         "Name:"           },
   { "version",      "Version:"        },
   { "author",       "Author:"         },
   { "copyright",    "Copyright:"      },
   { "email",        "Email:"          },
   { "date",         "Date:"           },
   { "requirements", "Requires:"       },
   { "requires",     "Requires:"       },
   { "license",      "License:"        },
   { "kernelversion","Kernel Version:" },
   { "platform",     "Platform:"       },
   { "platforms",    "Platforms:"      },
   { NULL, NULL }
};

ERROR ProcessInfo(struct Install *Self, struct XMLTag *XMLTag)
{
   UBYTE buffer[300];
   struct XMLTag *tag;
   OBJECTPTR text;
   LONG result, len, i;
   DOUBLE kernel;
   STRING platform, str;

   DPrintF("~Activate:","[Install] Processing <info> tag.");

   result = ERR_Failed;
   platform = NULL;

   ResetPage(Self);

   if (CreateObject(ID_TEXT, NULL, &text, NULL,
      FID_Owner|TLONG,        Self->ViewID,
      FID_TopMargin|TLONG,    3,
      FID_LeftMargin|TLONG,   3,
      FID_RightMargin|TLONG,  3,
      FID_BottomMargin|TLONG, 3,
      FID_Face|TSTRING,       glFontFace,
      FID_Flags|TLONG,        TXF_WORDWRAP,
      FID_Visible|TLONG,      FALSE,
      FID_VScroll|TLONG,      Self->VSViewID,
      FID_TabSize|TLONG,      28,
      TAGEND) != ERR_Okay) {
      StepBack();
      return(ERR_CreateObject);
   }

   for (i=0; InfoArray[i].Tag; i++) {
      if ((tag = FindTag(XMLTag->Child, InfoArray[i].Tag))) {

         for (tag=tag->Child; tag; tag=tag->Next) {
            if (!tag->Attributes[0].Name) {

               if (StrCompare("kernelversion", InfoArray[i].Tag, NULL, STR_MATCHLENGTH) IS ERR_Okay) {
                  /*** If this is the <kernelversion> tag, record the minimum required kernel version ***/
                  Self->prvKernelVersion = StrToFloat(tag->Attributes[0].Value);
                  break;
               }
               else if (StrCompare("installversion", InfoArray[i].Tag, NULL, STR_MATCHLENGTH) IS ERR_Okay) {
                  /*** If this is the <installversion> tag, record the minimum required install version ***/
                  Self->prvInstallVersion = StrToFloat(tag->Attributes[0].Value);
                  break;
               }
               else if (StrCompare("name", InfoArray[i].Tag, NULL, STR_MATCHLENGTH) IS ERR_Okay) {
                  /*** If this is the <name> tag, record it as the package's name ***/
                  Self->prvPackageName = StrClone(tag->Attributes[0].Value);
               }
               else if (StrCompare("platform", InfoArray[i].Tag, 8, NULL) IS ERR_Okay) {
                  if (StrSearch("All", tag->Attributes[0].Value, FALSE) IS -1) {
                     STRING myplatform;

                     myplatform = (STRING)(LONG)GetResource(RES_PLATFORM);

                     if (StrCompare("Windows", myplatform, 0, STR_MATCHLENGTH) IS ERR_Okay) {
                        if (StrSearch("Windows", tag->Attributes[0].Value, FALSE) IS -1) {
                           platform = tag->Attributes[0].Value;
                        }
                     }
                     else if (StrCompare("Linux", platform, 0, STR_MATCHLENGTH) IS ERR_Okay) {
                        if (StrSearch("Linux", tag->Attributes[0].Value, FALSE) IS -1) {
                           platform = tag->Attributes[0].Value;
                        }
                     }
                     else {
                        /*** Assume Athene/Athenyx ***/
                        if (StrSearch("Linux", tag->Attributes[0].Value, FALSE) IS -1) {
                           if (StrSearch("Athene", tag->Attributes[0].Value, FALSE) IS -1) {
                              if (StrSearch("Athenyx", tag->Attributes[0].Value, FALSE) IS -1) {
                                 platform = tag->Attributes[0].Value;
                              }
                           }
                        }
                     }
                  }
               }

               StrFormat(buffer, sizeof(buffer), "%s\t%s", InfoArray[i].Text, tag->Attributes[0].Value);

               acDataChannel(text, Self->Head.UniqueID, DATA_TEXT, 1, buffer, StrLength(buffer), 1);
            }
            break;
         }
      }
   }

   /*** Add the description ***/

   if ((tag = FindTag(XMLTag->Child, "description"))) {

      kernel = GetResource(RES_KERNELVERSION) + (((FLOAT)GetResource(RES_KERNELREVISION))/10);

      if (Self->prvKernelVersion > kernel + 0.0005) {  // The +0.0005 is to counter against floating point comparison errors (trust me!)
         ActionTags(AC_DataChannel, text, Self->Head.UniqueID, DATA_TEXT, 1, "", 1, 1);
         StrFormat(buffer, sizeof(buffer), "<item><font colour=\"#a02020\">This package requires kernel version %.1f and you are using v%.1f.  It is imperative that you upgrade your system if you wish to install this package without error.</font></item>", Self->prvKernelVersion, kernel);
         for (len=0; buffer[len]; len++);
         acDataChannel(text, Self->Head.UniqueID, DATA_XML, 1, buffer, len, 1);
      }

      if (Self->prvInstallVersion > VER_INSTALL + 0.0005) {  // The +0.0005 is to counter against floating point comparison errors (trust me!)
         ActionTags(AC_DataChannel, text, Self->Head.UniqueID, DATA_TEXT, 1, "", 1, 1);
         StrFormat(buffer, sizeof(buffer), "<item><font colour=\"#a02020\">This package requires version %.1f of the installer and you are using v%.1f.  You will need to upgrade the isntaller if you wish to install this package without error.</font></item>", Self->prvInstallVersion, VER_INSTALL);
         for (len=0; buffer[len]; len++);
         acDataChannel(text, Self->Head.UniqueID, DATA_XML, 1, buffer, len, 1);
      }

      if (platform) {
         ActionTags(AC_DataChannel, text, Self->Head.UniqueID, DATA_TEXT, 1, "", 1, 1);
         StrFormat(buffer, sizeof(buffer), "<item><font colour=\"#a02020\">This package will not run correctly on your system as it is limited to the following platform(s): %s</font></item>", platform);
         for (len=0; buffer[len]; len++);
         ActionTags(AC_DataChannel, text, Self->Head.UniqueID, DATA_XML, 1, buffer, len, 1);
      }

      acDataChannel(text, Self->Head.UniqueID, DATA_TEXT, 1, "", 1, 1);

      for (tag=tag->Child; tag; tag=tag->Next) {
         if (!tag->Attributes[0].Name) {
            str = tag->Attributes[0].Value;
            while ((*str) AND (*str <= 0x20)) str++; /* Skip leading whitespace */

            for (len=0; str[len]; len++);

            /*** Strip trailing whitespace ***/
            while (len > 0) {
               if (str[len-1] <= 0x20) len--;
               else break;
            }
            /*** Add content ***/
            acDataChannel(text, Self->Head.UniqueID, DATA_TEXT, 1, str, len, 1);
         }
      }
   }

   /*** "Click Next to Continue" ***/

   acDataChannel(text, Self->Head.UniqueID, DATA_XML, 1, txtClickNext, sizeof(txtClickNext), 1);

   /*** Update the display ***/

   SetFields(text, FID_Visible|TLONG, TRUE, TAGEND);

   ActionMsg(AC_Draw, Self->ViewID, NULL);

   StepBack();
   return(ERR_Okay);
}

/*****************************************************************************
** Internal: ProcessInstallation()
**
** The installation tag performs the actual installation of the files.  A
** variety of child tags are available for copying, deleting, moving and
** even running files on the system.
**
** EXAMPLE
** -------
** <installation>
**   <copy src="zterm.exe" dest="{dest}zterm" method="overwrite"/>
**   <merge/>
**   <run/>
**   <delete/>
**   <move/>
** </installation>
*/

ERROR ProcessInstallation(struct Install *Self, struct XMLTag *XMLTag)
{
   UBYTE intro[] = "<item><b><font colour=\"#202080\">Installation Ready</font></b></item><item/><item>The installation process is ready to start.  To go ahead with the installation of this software, please click the Next button.</item><item/>";

   DPrintF("~Activate:","[Install] Processing <installation> tag.");

   ResetPage(Self);

   if (CreateObject(ID_TEXT, NULL, &Self->prvOutput, NULL,
      FID_Owner|TLONG,        Self->ViewID,
      FID_LeftMargin|TLONG,   3,
      FID_RightMargin|TLONG,  3,
      FID_TopMargin|TLONG,    3,
      FID_BottomMargin|TLONG, 3,
      FID_Face|TSTRING,       glFontFace,
      FID_Flags|TLONG,        TXF_WORDWRAP,
      FID_VScroll|TLONG,      Self->VSViewID,
      FID_LineSpacing|TLONG,  2,
      TAGEND) != ERR_Okay) {
      StepBack();
      return(ERR_Failed);
   }

   acDataChannel(Self->prvOutput, Self->Head.UniqueID, DATA_XML, 1, intro, sizeof(intro), 1);

   StepBack();
   return(ERR_Okay);
}

/*****************************************************************************
** Internal: ProcessText()
*/

ERROR ProcessText(struct Install *Self, struct XMLTag *Tag)
{
   OBJECTPTR text;
   LONG i, len, j;
   BYTE nocontinue;

   DPrintF("~Activate:","[Install] Processing <text> tag.");

   ResetPage(Self);

   if (CreateObject(ID_TEXT, NULL, &text, NULL,
      FID_Owner|TLONG,        Self->ViewID,
      FID_LeftMargin|TLONG,   3,
      FID_RightMargin|TLONG,  3,
      FID_TopMargin|TLONG,    3,
      FID_BottomMargin|TLONG, 3,
      FID_Face|TSTRING,       glFontFace,
      FID_Flags|TLONG,        TXF_WORDWRAP,
      FID_VScroll|TLONG,      Self->VSViewID,
      TAGEND) != ERR_Okay) {

      StepBack();
      return(ERR_CreateObject);
   }

   /*** Check for a title in the <text> tag ***/

   nocontinue = FALSE;
   for (i=1; i < Tag->TotalAttributes; i++) {
      if (StrCompare("title", Tag->Attributes[i].Name, 0, STR_MATCHLENGTH) IS ERR_Okay) {
         for (len=0; Tag->Attributes[i].Value[len]; len++);

         {
            UBYTE h1[] = "<item><b><font colour=\"#202080\">";
            UBYTE h2[] = "</font></b></item><item/>";
            UBYTE buffer[sizeof(h1) + len + sizeof(h2) + 1];

            j = 0;
            for (len=0; h1[len]; len++) buffer[j++] = h1[len];
            for (len=0; Tag->Attributes[i].Value[len]; len++) buffer[j++] = Tag->Attributes[i].Value[len];
            for (len=0; h2[len]; len++) buffer[j++] = h2[len];
            buffer[j] = 0;
            acDataChannel(text, Self->Head.UniqueID, DATA_XML, 1, buffer, j, 1);
         }
      }
      else if (StrCompare("nocontinue", Tag->Attributes[i].Name, 0, STR_MATCHLENGTH) IS ERR_Okay) {
         nocontinue = TRUE;
      }
   }

   for (Tag=Tag->Child; Tag; Tag=Tag->Next) {
      if (!Tag->Attributes[0].Name) {
         for (len=0; Tag->Attributes[0].Value[len]; len++);
         acDataChannel(text, Self->Head.UniqueID, DATA_TEXT, 1, Tag->Attributes[0].Value, len, 1);
      }
   }

   if (nocontinue IS FALSE) {
      acDataChannel(text, Self->Head.UniqueID, DATA_XML, 1, txtClickNext, sizeof(txtClickNext), 1);
   }

   StepBack();
   return(ERR_Okay);
}
